-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
  onTagsChanged();
  DB.addHandler(DB.getPath(window.getDatabaseNode(), getTagName()), "onChildUpdate", onTagsChanged);
end

function onClose()
  DB.removeHandler(DB.getPath(window.getDatabaseNode(), getTagName()), "onChildUpdate", onTagsChanged);
end

function checkVisibility()
  local expand_button = getExpandButton();
  
  if expand_button then
    setVisible(expand_button.getValue() ~= 1);
    onTagsChanged();
  end
end

function onClickDown(button, x, y)
  if clickable then 
    local expand_button = getExpandButton();
    
    if expand_button then
      expand_button.setValue(1);
      setVisible(false);
    end
  end
end

function onTagsChanged()
  -- Set the effect summary string
  local sTags = getTagString();
  if sTags ~= "" then
    setValue(sTags);
  else
    setValue(nil);
  end
end

function getTagName()
  if tags then 
    return tags[1];
  else
    return "tags";
  end
end

function getExpandButton()
  if expand_button then 
    return window[expand_button[1]];
  else
    return nil;
  end
end

function getTagString()
  -- Start with an empty tag list string
  local aOutputTags = {};
  
  -- Iterate through each tag
  local aSorted = {};
  for _,nodeChild in pairs(DB.getChildren(window.getDatabaseNode(), getTagName())) do
    table.insert(aSorted, nodeChild);
  end
  table.sort(aSorted, function (a, b) return a.getName() < b.getName() end);
  for _,v in pairs(aSorted) do
    local sLabel = DB.getValue(v, "name", "");
    local sScore = DB.getValue(v, "score", "");

    if string.len(sScore) > 0 then 
      sLabel = sLabel.." "..sScore;
    end

    if string.len(sLabel) > 0 then 
      table.insert(aOutputTags, sLabel);
    end
  end
  
  -- Return the final effect list string
  return table.concat(aOutputTags, ", ");
end

