-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

local expanded;

function onInit()
  if super and super.onInit then
    super.onInit();
  end
  expanded = window.expanded;
end

function getOpeningLabel()
  return "action";
end

function getExpanded()
  return expanded.getValue();
end

function setExpanded(v)
  expanded.setValue(v);
end

function getActor()
  return ActorManager.getActorFromCT(self.getNode());
end

function onButtonPress()
  if not super.onButtonPress() then
    if self.getLevel() ~= 2 or getExpanded() > 0 then
      action();
      setExpanded(0);
    else
      setExpanded(1);
    end
    window.updateDisplay();
  end
end

function getNode()
  return super.getNode();
end

function getOpening()
  return super.getOpening();
end

function updateSR()
  if getLevel() == 2 then
    local sr = self.getNode().createChild("initresult", "number").getValue();
  
    sr = sr - self.getSrExpenditure(); 
    self.getNode().getChild("initresult").setValue(sr);
    self.open("*");
  end
end

function getLevel()
  if window.level then 
    return window.level.getValue();
  else
    return 0;
  end
end

function getWeapon(type)
  local control = "";
  
  if type == "defence" then
    control = "defence_ready";
  elseif type == "ranged" or type == "melee" or type == "attack" then
    control = "attack_ready";
  else
    return { reach=0, attack_sr=3, defence_sr=5 };
  end
  
  local weapons = window.getDatabaseNode().getChild("weaponlist");
   
  if not weapons and window.link and window.link.getTargetDatabaseNode() then
    weapons = window.link.getTargetDatabaseNode().getChild("weaponlist");
  end 
   
  if weapons then
    for _,weapon in pairs(weapons.getChildren()) do
      if weapon.getChild(control).getValue() > 0 then
        return { name=weapon.getChild("name").getValue(),
          reach=weapon.getChild("reach").getValue(), 
          attack_sr=weapon.getChild("attack_sr").getValue(), 
          defence_sr=weapon.getChild("defence_sr").getValue(),
          special_opening=weapon.createChild("special_opening", "string").getValue() }; 
      end
    end
  end

  if window.attacks then
    for _,weapon in ipairs(window.attacks.getWindows()) do
      if weapon[control].getValue() > 0 then
        return { name=weapon.name.getValue(),
          reach=weapon.reach.getValue(), 
          attack_sr=weapon.attack_sr.getValue(), 
          defence_sr=weapon.defence_sr.getValue(),
          special_opening=weapon.special_opening.getValue() }; 
      end
    end
  end

  return { reach=0, attack_sr=3, defence_sr=5 };
end

function canAct()
  return self.getLevel() < 2 or (string.len(getOpening()) > 0 
    and (getOpening() == getOpeningLabel() or action))
end

function action()
  local weapon = self.getWeapon(self.getOpeningLabel());
  local rRoll = { sType = "action", 
        sDesc = Interface.getString("message_"..self.getOpeningLabel()), 
        aDice = {}, 
        nMod = self.getSrExpenditure() };
        
  if self.canAct() then
    if window.getDatabaseNode().getChild("skill") then 
      rRoll.sDesc = window.getDatabaseNode().getChild("skill").getValue();
      if skill == nil or skill:len() == 0 then
        rRoll.sDesc = Interface.getString("message_"..self.getOpeningLabel());
      end
    end
    rRoll.sType = "attack";
    rRoll.aDice = GameSystem.getPercentileDice();
    rRoll.nMod = 0;
  end
  
  if weapon and weapon.name then
    rRoll.sDesc = rRoll.sDesc.." with "..weapon.name;
  end
  if window.getDatabaseNode().getChild("trait") then 
    local trait = window.getDatabaseNode().getChild("trait").getValue();
    if trait and trait ~= "" then
      trait = " ["..trait.."]";
    else
      trait = "";
    end
    rRoll.sDesc = rRoll.sDesc..trait;
  end
  
  if getLevel() == 2 then
    local cost = self.getSrExpenditure();
    self.updateSR();
    if cost > 0 then
      rRoll.sDesc = rRoll.sDesc.." ("..tostring(cost).." sr)"
    end
  end
  if #rRoll.aDice > 0 then 
    AttackManager.performRoll(nil, getActor(), rRoll);
  else
    ChatManager.Message(rRoll.sDesc,
        not (User.isHost() and OptionsManager.isOption("REVL", "off")), 
        getActor());
  end
end
