-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
	-- Set the displays to what should be shown
	setTargetingVisible();
	setSpacingVisible();
	setEffectsVisible();
	setAttacksVisible();

	-- Acquire token reference, if any
	linkToken();
	
	-- Set up the PC links
	onLinkChanged();
	
	-- Update the displays
	onFactionChanged();
	
	-- Register the deletion menu item for the host
	registerMenuItem(Interface.getString("list_menu_deleteitem"), "delete", 6);
	registerMenuItem(Interface.getString("list_menu_deleteconfirm"), "delete", 6, 7);

  combatReady(link.getTargetDatabaseNode());
  
  -- DB.addHandler(opening.getDatabaseNode().getPath(), "onUpdate", updateDisplay);
  DB.addHandler(link.getTargetDatabaseNode().createChild("opening", "string").getPath(), "onUpdate", updateDisplay);
  
  updateDisplay();
end

function onClose()
  -- DB.removeHandler(opening.getDatabaseNode().getPath(), "onUpdate", updateDisplay);
  DB.removeHandler(link.getTargetDatabaseNode().getChild("opening").getPath(), "onUpdate", updateDisplay);
end

function toggleAttacks()
	if not enableglobaltoggle then
		return;
	end
	
	local attackson = window.button_global_attacks.getValue();
	for _,v in pairs(getWindows()) do
			v.activateattacks.setValue(attackson);
	end
end

function combatReady(character)
  local health = character.createChild("health", "number").getValue();
  local basehealth = character.createChild("basehealth", "number").getValue();
  local opening = character.createChild("opening", "string").getValue();

  if opening == nil then
    character.getChild("opening").setValue("");
  end
  if basehealth > 0 and health == nil then
    character.getChild("health").setValue(basehealth);
  end
end

-- from CoreRPG 
function onVisibilityToggle()
	local anyVisible = 0;
	for _,v in pairs(getWindows()) do
		if (v.friendfoe.getStringValue() ~= "friend") and (v.tokenvis.getValue() == 1) then
			anyVisible = 1;
		end
	end
	
	enablevisibilitytoggle = false;
	window.button_global_visibility.setValue(anyVisible);
	enablevisibilitytoggle = true;
end

-- from CoreRPG 
function updateDisplay()
	local sFaction = friendfoe.getStringValue();
	local sOpening = link.getTargetDatabaseNode().createChild("opening", "string").getValue();
  local nLevel = level.getValue();

  if string.len(sOpening) == 0 then
    sOpening = nil;
  end
  if nLevel ~= nil and nLevel ~= 2 then 
    sOpening = "generic"
  end

  --power_opening.setVisible(sOpening == nil or sOpening == "power");
  ranged_opening.setVisible(expanded.getValue() > 0 );
  melee_opening.setVisible(sOpening == nil or sOpening == "melee" or expanded.getValue() > 0 );
  generic_opening.setVisible(sOpening == nil or sOpening == "generic");
  action.setVisible(sOpening ~= nil and nLevel == 2 and expanded.getValue() > 0 and sOpening ~= "generic");
  defence.setVisible(sOpening ~= nil and nLevel == 2 and expanded.getValue() > 0);
  wait.setVisible(sOpening ~= nil and nLevel == 2 and expanded.getValue() > 0);

	if DB.getValue(getDatabaseNode(), "active", 0) == 1 and level.getValue() ~= 2 then
		name.setFont("sheetlabel");
		
		active_spacer_top.setVisible(true);
		active_spacer_bottom.setVisible(true);
		
		if sFaction == "friend" then
			setFrame("ctentrybox_friend_active");
		elseif sFaction == "neutral" then
			setFrame("ctentrybox_neutral_active");
		elseif sFaction == "foe" then
			setFrame("ctentrybox_foe_active");
		else
			setFrame("ctentrybox_active");
		end
	else
		name.setFont("sheettext");
		
		active_spacer_top.setVisible(false);
		active_spacer_bottom.setVisible(false);
		
		if sFaction == "friend" then
			setFrame("ctentrybox_friend");
		elseif sFaction == "neutral" then
			setFrame("ctentrybox_neutral");
		elseif sFaction == "foe" then
			setFrame("ctentrybox_foe");
		else
			setFrame("ctentrybox");
		end
	end
end

-- from CoreRPG 
function linkToken()
	local imageinstance = token.populateFromImageNode(tokenrefnode.getValue(), tokenrefid.getValue());
	if imageinstance then
		TokenManager.linkToken(getDatabaseNode(), imageinstance);
	end
end

-- from CoreRPG 
function onMenuSelection(selection, subselection)
	if selection == 6 and subselection == 7 then
		delete();
	end
end

-- from CoreRPG 
function delete()
	local node = getDatabaseNode();
	if not node then
		close();
		return;
	end
	
	-- Remember node name
	local sNode = node.getNodeName();
	
	-- Clear any effects and wounds first, so that saves aren't triggered when initiative advanced
	effects.reset(false);
	
	-- Move to the next actor, if this CT entry is active
	if DB.getValue(node, "active", 0) == 1 then
		CombatManager.nextActor();
	end

	-- Delete the database node and close the window
	node.delete();

	-- Update list information (global subsection toggles)
	windowlist.onVisibilityToggle();
	windowlist.onEntrySectionToggle();
end

-- from CoreRPG 
function onLinkChanged()
	-- If a PC, then set up the links to the char sheet
	local sClass, sRecord = link.getValue();
	if sClass == "charsheet" then
		linkPCFields();
		name.setLine(false);
	end
end

-- from CoreRPG 
function onFactionChanged()
	-- Update the entry frame
	updateDisplay();

	-- If not a friend, then show visibility toggle
	if friendfoe.getStringValue() == "friend" then
		tokenvis.setVisible(false);
	else
		tokenvis.setVisible(true);
	end
end

-- from CoreRPG 
function onVisibilityChanged()
	TokenManager.updateVisibility(getDatabaseNode());
	windowlist.onVisibilityToggle();
end

function linkPCFields()
	local nodeChar = link.getTargetDatabaseNode();

	if nodeChar then
		name.setLink(nodeChar.createChild("name", "string"), true);
    initresult.setLink(nodeChar.createChild("initresult", "number"), false);
		health.setLink(nodeChar.createChild("health", "number"), false);
    -- wounds.setLink(nodeChar.createChild("wounds", "number"), false);
    rp.setLink(nodeChar.createChild("rp", "number"), false);
    toughness.setLink(nodeChar.createChild("toughness", "number"), true);
    armour.setLink(nodeChar.createChild("armour", "string"), true);
    -- atk.setLink(nodeChar.createChild("attacks", "string"), true);
    opening.setLink(nodeChar.createChild("opening", "string"), false);
    reach.setLink(nodeChar.createChild("reach", "number"), false);
    
    --- Debug.console(nodeChar.getChild("weaponlist"));
    attacks.setDatabaseNode(nodeChar.createChild("weaponlist", "node"));
    
	end
end


--
-- SECTION VISIBILITY FUNCTIONS
--
function setAttacksVisible()
	local v = activateattacks.getValue() == 1;

	attackicon.setVisible(v);
	attacks.setVisible(v);
	frame_attacks.setVisible(v);
end

-- from CoreRPG
function setTargetingVisible()
	local v = activatetargeting.getValue() == 1;

  targetingicon.setVisible(v);
	sub_targeting.setVisible(v);
	frame_targeting.setVisible(v);
	target_summary.onTargetsChanged();
end

-- from CoreRPG
function setSpacingVisible()
	local v = activatespacing.getValue() == 1;
	
	spacingicon.setVisible(v);
	toughness.setVisible(v);
	toughnesslabel.setVisible(v);
  armour.setVisible(v);
  armourlabel.setVisible(v);
	frame_spacing.setVisible(v);
end

-- From CoreRPG
function setEffectsVisible()
	local v = false;
	if activateeffects.getValue() == 1 then
		v = true;
	end
	
	effecticon.setVisible(v);
	
	effects.setVisible(v);
	effects_iadd.setVisible(v);
	for _,w in pairs(effects.getWindows()) do
		w.idelete.setValue(0);
	end

	frame_effects.setVisible(v);

	effect_summary.onEffectsChanged();
end

-- Rd100
function setLevel(nLevel)
  level.setValue(nLevel);

  -- wounds.setVisible(nLevel == 2);
  health.setVisible(nLevel == 2);
  initresult.setVisible(nLevel > 0);
  
  moving.setVisible(nLevel == 3);

  activateeffects.setVisible(nLevel > 0);
  activatespacing.setVisible(nLevel == 2);
  activateattacks.setVisible(nLevel > 0);
  
  if nLevel == 0 then
    activateattacks.setValue(0);
    activateeffects.setValue(0);
  end
  if nLevel < 2 then
    activatespacing.setValue(0);
  end
  setAttacksVisible();
  setSpacingVisible();
  setEffectsVisible();
  windowlist.onEntrySectionToggle();
  
  active.setVisible(nLevel ~= 2);
  
  expanded.setValue(0);
  
  updateDisplay(); 
end

function onDrop(x, y, draginfo)
  -- if draginfo.isType("shortcut") then
    -- trait.setValue(draginfo.getDescription());
    -- getDatabaseNode().getChild("trait").setValue(draginfo.getDescription());
    -- return true;
  -- end
end

function update()
  updateDisplay();
end

