-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function getActor()
  local node = window.getDatabaseNode();
  
  return ActorManager.getActorFromCT(node);
end

function onButtonPress()
  if window.opening and string.len(window.opening.getValue()) == 0 then
    Debug.console("Setting opening to " .. self.getOpeningValue());
      
    if super and super.onButtonPress then
      super.onButtonPress();
    end
    
    if not window.opening.getDatabaseNode().isOwner() then
      Debug.console("Warning: not owner, using link");
      self.getNode().createChild("opening", "string").setValue(self.getOpeningValue());
    end
    window.opening.setValue(self.getOpeningValue());
    window.opened.setValue(nil);
  else
    action();
  end
end

function getOpeningValue()
  return nil;
end

function updateSR()
  local sr = self.getNode().createChild("initresult", "number").getValue();

  sr = sr - self.getSrExpenditure(); 
  self.getNode().getChild("initresult").setValue(sr);
  window.opened.setValue("*");
end

function getNode()
  return window.link.getTargetDatabaseNode();
end

function getSrExpenditure()
  return 5;
end

function getWeapon()
  if window.link then
    local weapons = window.link.getTargetDatabaseNode().getChild("weaponlist");
    
    for _,weapon in pairs(weapons.getChildren()) do
      if weapon.getChild("equipped").getValue() > 0 then
        return { reach=weapon.getChild("reach").getValue(), 
          attack_sr=weapon.getChild("attack_sr").getValue(), 
          defence_sr=weapon.getChild("defence_sr").getValue(),
          special_opening=weapon.getChild("special_opening").getValue() }; 
      end
    end
  end
  if window.attacks then
    for _,weapon in ipairs(window..attacks.getWindows()) do
      if weapon.equipped.getValue() > 0 then
        return { reach=weapon.reach.getValue(), 
          attack_sr=weapon.attack_sr.getValue(), 
          defence_sr=weapon.defence_sr.getValue(),
          special_opening=weapon.special_opening.getValue() }; 
      end
    end
  end
  return { reach=0, attack_sr=3, defence_sr=2 };
end

function action()
  local trait = "";
  local mod = 0;
  local skill = "Roll "..self.getOpeningValue();
  
  if window.getDatabaseNode().getChild("skill") then 
    skill = window.getDatabaseNode().getChild("skill").getValue();
    if skill == nil or skill:len() == 0 then
      skill = "Roll "..self.getOpeningValue();
    end
  end
  if window.getDatabaseNode().getChild("trait") then 
    trait = window.getDatabaseNode().getChild("trait").getValue();
    if trait and trait ~= "" then
      trait = " ["..trait.."]";
    else
      trait = "";
    end
    -- mod = 30;
  end
  local rRoll = { sType = "attack", 
        sDesc = skill..trait, 
        aDice = GameSystem.getPercentileDice(), 
        nMod = mod};
        
  self.updateSR(); 
  AttackManager.performRoll(nil, getActor(), rRoll);
end
