-- 
-- Please see the license.html file included with this distribution for 
-- attribution and copyright information.
--

function onInit()
  Debug.console("RD100_Combat.onInit");
	CombatManager.setCustomSort(sortfunc);
	--CombatManager.setCustomDrop(onDrop);

  CombatManager.setCustomAddNPC(addNPC);
  CombatManager.setCustomAddPC(addPC);
	--CombatManager.setCustomNPCSpaceReach(getNPCSpaceReach);

	CombatManager.setCustomRoundStart(onRoundStart);
  CombatManager.setCustomCombatReset(resetInit);
  CombatManager.setCustomAddBattle(rd100AddBattle);
end

function rd100AddBattle(nodeBattle) 
  
  -- Cycle through the NPC list, and add them to the tracker
  for _, vNPCItem in pairs(DB.getChildren(nodeBattle, LibraryData.getCustomData("battle", "npclist") or "npclist")) do
    -- Get link database node
    local nodeNPC = nil;
    local sClass, sRecord = DB.getValue(vNPCItem, "link", "", "");
    if sRecord ~= "" then
      nodeNPC = DB.findNode(sRecord);
    end
    local sName = DB.getValue(vNPCItem, "name", "");
    
    if nodeNPC then
      local aPlacement = {};
      for _,vPlacement in pairs(DB.getChildren(vNPCItem, "maplink")) do
        local rPlacement = {};
        local _, sRecord = DB.getValue(vPlacement, "imageref", "", "");
        rPlacement.imagelink = sRecord;
        rPlacement.imagex = DB.getValue(vPlacement, "imagex", 0);
        rPlacement.imagey = DB.getValue(vPlacement, "imagey", 0);
        table.insert(aPlacement, rPlacement);
      end
      
      local nCount = DB.getValue(vNPCItem, "count", 0);
      for i = 1, nCount do
        local nodeEntry = addNPC(sClass, nodeNPC, sName);
        if nodeEntry then
          local sFaction = DB.getValue(vNPCItem, "faction", "");
          if sFaction ~= "" then
            DB.setValue(nodeEntry, "friendfoe", "string", sFaction);
          end
          local sToken = DB.getValue(vNPCItem, "token", "");
          if sToken == "" or not Interface.isToken(sToken) then
            local sLetter = StringManager.trim(sName):match("^([a-zA-Z])");
            if sLetter then
              sToken = "tokens/Medium/" .. sLetter:lower() .. ".png@Letter Tokens";
            else
              sToken = "tokens/Medium/z.png@Letter Tokens";
            end
          end
          if sToken ~= "" then
            DB.setValue(nodeEntry, "token", "token", sToken);
            
            if aPlacement[i] and aPlacement[i].imagelink ~= "" then
              TokenManager.setDragTokenUnits(DB.getValue(nodeEntry, "space"));
              local tokenAdded = Token.addToken(aPlacement[i].imagelink, sToken, aPlacement[i].imagex, aPlacement[i].imagey);
              TokenManager.endDragTokenWithUnits(nodeEntry);
              if tokenAdded then
                TokenManager.linkToken(nodeEntry, tokenAdded);
              end
            end
          end
        else
          ChatManager.SystemMessage(Interface.getString("ct_error_addnpcfail") .. " (" .. sName .. ")");
        end
      end
    else
      ChatManager.SystemMessage(Interface.getString("ct_error_addnpcfail2") .. " (" .. sName .. ")");
    end
  end
  
  Interface.openWindow("combattracker_host", "combattracker");
end

--
-- COMBAT TRACKER SORT
--

-- NOTE: Lua sort function expects the opposite boolean value compared to built-in FG sorting
function sortfunc(node1, node2)
	local nValue1 = DB.getValue(node1, "initresult", 0);
	local nValue2 = DB.getValue(node2, "initresult", 0);
	if nValue1 ~= nValue2 then
		return nValue1 > nValue2;
	end
	
	local sValue1 = DB.getValue(node1, "name", "");
	local sValue2 = DB.getValue(node2, "name", "");
	if sValue1 ~= sValue2 then
		return sValue1 < sValue2;
	end

	return node1.getNodeName() < node2.getNodeName();
end

--
-- PER ROUND INITIATIVE
--

function onRoundStart(nRound)
	if OptionsManager.isOption("HRIR", "on") then
		-- rollInit();
	end
  resetInit();
  return nil;
end

--
-- DROP HANDLING
--

function onDrop(rSource, rTarget, draginfo)
	local sDragType = draginfo.getType();

	-- Effect targeting
	if sDragType == "effect_targeting" then
		if User.isHost() then
			onEffectTargetingDrop(rSource, rTarget, draginfo);
			return true;
		end
	end
end

function onEffectTargetingDrop(rSource, rTarget, draginfo)
	local sTargetCT = ActorManager.getCTNodeName(rTarget);
	if sTargetCT ~= "" then
		local sRefClass, sEffectNode = draginfo.getShortcutData();
		if sRefClass and sEffectNode then
			if sRefClass == "ct_effect" then
				EffectManager.addEffectTarget(sEffectNode, sTargetCT);
			end
		end
	end
end

--
-- ADD FUNCTIONS
--

function getNPCSpaceReach(nodeNPC)
	local nSpace = GameSystem.GameSystem.getDistanceUnitsPerGrid();
	local nReach = nSpace;
	
	local sSize = DB.getValue(nodeNPC, "size", "");
	if sSize:lower() == "large" then
		nSpace = nSpace * 2;
		nReach = nReach * 2;
	end

	return nSpace, nReach;
end

function addNPC(sClass, nodeNPC, sName)
  local nodeEntry, nodeLastMatch = CombatManager.addNPCHelper(nodeNPC, sName);

  local bMonster = true;
  if sClass == "monster" then
    bMonster = true;
    CampaignDataManager2.convertMonsterToNPC(nodeNPC, nodeEntry);
  else
    bMonster = (DB.getValue(nodeEntry, "monster", "") ~= "false");
  end
  
  DB.setValue(nodeEntry, "initresult", "number", 0);
  DB.setValue(nodeEntry, "opening", "string", "");
  DB.setValue(nodeEntry, "opened", "string", nil);

  return nodeEntry;
end

function addVehicle(sClass, nodeNPC, sName)
  local nodeEntry, nodeLastMatch = CombatManager.addNPCHelper(nodeNPC, sName);

  DB.setValue(nodeEntry, "initresult", "number", 0);
  DB.setValue(nodeEntry, "opening", "string", "");
  DB.setValue(nodeEntry, "opened", "string", nil);
  
    -- Set up the CT specific information
  DB.setValue(nodeEntry, "link", "windowreference", "vehicle", "");
  if not bIsCTNPC then
    DB.setValue(nodeEntry, "sourcelink", "windowreference", "vehicle", nodeNPC.getPath());
  end

  return nodeEntry;
end

function addPC(nodePC)
  -- Parameter validation
  if not nodePC then
    return;
  end

  -- Create a new combat tracker window
  local nodeEntry = DB.createChild(CombatManager.CT_LIST);
  if not nodeEntry then
    return;
  end
  
  -- Set up the CT specific information
  DB.setValue(nodeEntry, "link", "windowreference", "charsheet", nodePC.getNodeName());
  DB.setValue(nodeEntry, "friendfoe", "string", "friend");

  local sToken = DB.getValue(nodePC, "token", nil);
  if not sToken or sToken == "" then
    sToken = "portrait_" .. nodePC.getName() .. "_token"
  end
  DB.setValue(nodeEntry, "token", "token", sToken);

  DB.setValue(nodeEntry, "initresult", "number", 0);
  DB.setValue(nodeEntry, "opening", "string", "");
  DB.setValue(nodeEntry, "opened", "string", nil);
  
end

--
-- RESET FUNCTIONS
--

function resetInit()
  local list = DB.findNode(CombatManager.CT_LIST);
  local level = list.createChild("..level", "number").getValue();
  
  if level == 2 then
    for _, vChild in pairs(list.getChildren()) do
      DB.setValue(vChild, "initresult", "number", 0);
      DB.setValue(vChild, "opening", "string", "");
      DB.setValue(vChild, "opened", "string", nil);
    end
  elseif level == 1 then
    for _, vChild in pairs(list.getChildren()) do
      DB.setValue(vChild, "initresult", "number", DB.getValue(vChild, "rp", 0));
    end
  end
end

function resetEffects()
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local nodeEffects = vChild.getChild("effects");
		if nodeEffects then
			for _, vEffect in pairs(nodeEffects.getChildren()) do
				vEffect.delete();
			end
		end
	end
end

function clearExpiringEffects(bShort)
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local nodeEffects = vChild.getChild("effects");
		if nodeEffects then
			for _, vEffect in pairs(nodeEffects.getChildren()) do
				local sLabel = DB.getValue(vEffect, "label", "");
				local nDuration = DB.getValue(vEffect, "duration", 0);
				
				if nDuration ~= 0 or sLabel == "" then
					if bShort then
						if nDuration > 50 then
							DB.setValue(vEffect, "duration", "number", nDuration - 50);
						else
							vEffect.delete();
						end
					else
						vEffect.delete();
					end
				end
			end
		end
	end
end

-- redundant
function rest(bShort)
	CombatManager.resetInit();
	clearExpiringEffects(bShort);
	
	if not bShort then
		for _,vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
			local sClass, sRecord = DB.getValue(vChild, "link", "", "");
			if sClass == "charsheet" and sRecord ~= "" then
				local nodePC = DB.findNode(sRecord);
				if nodePC then
					CharManager.rest(nodePC);
				end
			end
		end
	end
end

function rollEntryInit(nodeEntry)
	if not nodeEntry then
		return;
	end
	
	local nInitDieSides = 10;
	local sOptHRID = OptionsManager.getOption("HRID");
	if sOptHRID == "100" then
		nInitDieSides = 100;
	elseif sOptHRID == "20" then
		nInitDieSides = 20;
	end

	-- For PCs, we always roll unique initiative
	local sClass, sRecord = DB.getValue(nodeEntry, "link", "", "");
	if sClass == "charsheet" then
		local nInitResult = math.random(nInitDieSides);
		local rActor = ActorManager.getActor("pc", sRecord);
		if rActor then
			nInitResult = nInitResult + DB.getValue(ActorManager.getCreatureNode(rActor), "initiative.total", 0);
		end
		
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
		return;
	end
	
	-- For NPCs, if NPC init option is not group, then roll unique initiative
	local sOptHRIN = OptionsManager.getOption("HRIN");
	if sOptHRIN ~= "group" then
		local nInitResult = math.random(nInitDieSides);
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
		return;
	end

	-- For NPCs with group option enabled
	
	-- Get the entry's database node name and creature name
	local sStripName = CombatManager.stripCreatureNumber(DB.getValue(nodeEntry, "name", ""));
	if sStripName == "" then
		local nInitResult = math.random(nInitDieSides);
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
		return;
	end
		
	-- Iterate through list looking for other creature's with same name
	local nLastInit = nil;
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		if vChild.getName() ~= nodeEntry.getName() then
			local sTemp = CombatManager.stripCreatureNumber(DB.getValue(vChild, "name", ""));
			if sTemp == sStripName then
				local nChildInit = DB.getValue(vChild, "initresult", 0);
				if nChildInit ~= -10000 then
					nLastInit = nChildInit;
				end
			end
		end
	end
	
	-- If we found similar creatures, then match the initiative of the last one found
	if nLastInit then
		DB.setValue(nodeEntry, "initresult", "number", nLastInit);
	else
		local nInitResult = math.random(nInitDieSides);
		DB.setValue(nodeEntry, "initresult", "number", nInitResult);
	end
end

function rollInit(sType)
	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local bRoll = true;
		if sType then
			local sClass,_ = DB.getValue(vChild, "link", "", "");
			if sType == "npc" and sClass == "charsheet" then
				bRoll = false;
			elseif sType == "pc" and sClass ~= "charsheet" then
				bRoll = false;
			end
		end
		
		if bRoll then
			DB.setValue(vChild, "initresult", "number", -10000);
		end
	end

	for _, vChild in pairs(DB.getChildren(CombatManager.CT_LIST)) do
		local bRoll = true;
		if sType then
			local sClass,_ = DB.getValue(vChild, "link", "", "");
			if sType == "npc" and sClass == "charsheet" then
				bRoll = false;
			elseif sType == "pc" and sClass ~= "charsheet" then
				bRoll = false;
			end
		end
		
		if bRoll then
			rollEntryInit(vChild);
		end
	end
end

--
-- PARSE CT ATTACK LINE
--

-- redundant
function parseAttackLine(rActor, sText)
	local rAttackRolls = {};
	local rDamageRolls = {};

	sText = sText:gsub("�", "-");

	local aClauses = {};
	local nIndex = 1;
	local nStart, nEnd, nStart2, nEnd2, nStart3, nEnd3;
	local sPhrase;
	while nIndex < #sText do
		nStart, nEnd = string.find(sText, "%s+or%s+", nIndex);
		nStart2, nEnd2 = string.find(sText, "%s*;%s*", nIndex);
		nStart3, nEnd3 = string.find(sText, "%)%s*,%s*", nIndex);
		nStart4, nEnd4 = string.find(sText, "%s+and%s+", nIndex);
		
		if nStart2 and (not nStart or nStart > nStart2) then
			nStart = nStart2;
			nEnd = nEnd2;
		end
		if nStart3 and (not nStart or nStart > nStart3) then
			nStart = nStart3 + 1;
			nEnd = nEnd3;
		end
		if nStart4 and (not nStart or nStart > nStart4) then
			nStart = nStart4;
			nEnd = nEnd4;
		end
		
		if nStart then
			sPhrase = string.sub(sText, nIndex, nEnd - 1);
		else
			sPhrase = string.sub(sText, nIndex);
		end
		
		nPhraseStart, sAll, sAttackCount, sAttackLabel, nDamageStart, sDamage, nDamageEnd, nPhraseEnd = string.match(sPhrase, '()((%d*) ?([%w%s%[%]%+%-]*) %(()([^%)]*)()%))()');
		if nPhraseStart then
			local rAttack = {};
			rAttack.startpos = nIndex + nPhraseStart - 1;
			rAttack.endpos = nIndex + nPhraseEnd - 1;
			
			local rDamage = {};
			rDamage.startpos = nIndex + nDamageStart - 1;
			rDamage.endpos = nIndex + nDamageEnd - 1;
			
			-- Clean up the attack count field (i.e. magical weapon bonuses up front, no attack count)
			local nAttackCount = 1;
			if #sAttackCount then
				nAttackCount = tonumber(sAttackCount) or 1;
				if nAttackCount < 1 then
					nAttackCount = 1;
				end
			end

			-- Watch for touch attacks
			local aLabelWords = StringManager.parseWords(sAttackLabel:lower());
			if StringManager.contains(aLabelWords, "touch") then
				rAttack.touch = true;
			end
			
			-- Decode any attack bonus
			local nBonusStart, sAttackModifier = string.match(sAttackLabel, "()%s+([%+%-]?%d+)$");
			if nBonusStart then
				sAttackLabel = sAttackLabel:sub(1, nBonusStart);
			end
			
			-- Capitalize first letter of label
			sAttackLabel = StringManager.capitalize(sAttackLabel);
			
			rAttack.label = sAttackLabel;
			rAttack.count = nAttackCount;
			rAttack.modifier = sAttackModifier;
			
			rDamage.label = sAttackLabel;
			local aDamageDice, nDamageMod = StringManager.convertStringToDice(sDamage)
			rDamage.dice = aDamageDice;
			rDamage.modifier = nDamageMod;
			
			-- Add to roll list
			table.insert(rAttackRolls, rAttack);
			table.insert(rDamageRolls, rDamage);
		end
		
		if nStart then
			nIndex = nEnd + 1;
		else
			nIndex = #sText;
		end
	end
	
	return rAttackRolls, rDamageRolls;
end
